FlamingLoot = LibStub("AceAddon-3.0"):NewAddon("FlamingLoot", "AceConsole-3.0", "AceEvent-3.0")

--VARIABLES
local lootTable = {} --[#].name, rollopts, texture, link, selec, msg; items added in StartLooting, removed in RollonItem
local currentitem = 1
local lootcount = 0
local rollingopen = false
local playerclass = UnitClass("player")
local optsdisplay = {
	["n"] = "need",
	["i"] = "minor upgrade",
	["s"] = "sidegrade",
	["o"] = "offspec",
	["g"] = "greed",
	["p"] = "pass",
}

local fra = CreateFrame("Frame", "FLR_ItemCachingFrame")
local interval = 0

local defaults = {
	profile = {
		masterlooter = "masterlooter",
		standings = "no standings stored",
		standingstime = 0,
	},
}

local options = {
	name = "Raider Commands",
    handler = FlamingLoot,
    type = "group",
	args = {
		standings = {
			name = "standings", type = "execute",
			desc = "View standings.  Can also be accessed with |cffffff78/fls|r",
			func = "ShowStandings",
			guiHidden = true,
		},
		update = {
			name = "update", type = "execute",
			desc = "Update your standings.  Only use this if you missed a raid or part of one - standings are automatically broadcast to the raid whenever they are updated.  Requires your masterlooter to be online.",
			func = "GetStandings",
			guiHidden = true,
		},
	}
}
--INITIALIZE & ENABLE
function FlamingLoot:OnInitialize()
	RegisterAddonMessagePrefix("flamingloot")
	self.db = LibStub("AceDB-3.0"):New("FlamingLootDB", defaults)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("FL", options, {"fl"})
end

function FlamingLoot:OnEnable()
	self:RegisterEvent("CHAT_MSG_ADDON")
	self:RegisterChatCommand("manflower", "InconspicuousFunction")
	RaiderStandings:SetScale(0.8)
	FLLootRollFrame:SetScale(0.8)
	RaiderStandingsTitle:SetText("Standings")
end

--MAIN FUNCS
function FlamingLoot:StartLooting(sender, param)
	--format: "item:id=rolltype; item name=rolltype; etc"
	FLR_ItemCachingFrame:Show()
	lootcount = 0
	--self:Print("looting started")
	rollingopen = true
	self.db.profile.masterlooter = sender
	for itemid, rollopts in string.gmatch(param, "(%d+)=(%a+);") do
		lootcount = lootcount+1
		if not lootTable[lootcount] then --only make a new table if we haven't already
			lootTable[lootcount] = {}
		end
		lootTable[lootcount]["itemid"] = tonumber(itemid)
		lootTable[lootcount]["rollopts"] = rollopts
		--self:Print("item number "..lootcount.." - id is "..lootTable[lootcount]["itemid"]..", roll opts are "..lootTable[lootcount]["rollopts"]) --DEBUG
	end
end

local done = false	
local function GetLootInfo(self, elapsed) --frame onupdate method from epgp
	interval = interval + elapsed
	if interval>.5 and lootTable[1].itemid then
		done = true
		for i, itemtable in pairs(lootTable) do
			if not lootTable[i].name then 
				--FlamingLoot:Print("looking up item id "..itemtable.itemid)
				local test = GetItemInfo(itemtable.itemid)
				if not test then
					done = false
				else
					itemtable["name"] = select(1, test)
					--FlamingLoot:Print("Name is "..itemtable.name)
				end
			end
		end
		if done==true then
			--FlamingLoot:Print("done!")
			fra:Hide()
			FlamingLoot:ProcessLoot()
		end
		interval = 0
	end
end
fra:SetScript("OnUpdate", GetLootInfo)
fra:Hide()

function FlamingLoot:ProcessLoot()
	for i=1, lootcount do
		local iname, ilink, _, ilvl, _, class, subtype, _, equipSlot, texture, _ = GetItemInfo(lootTable[i].itemid)
		--self:Print("item number "..i.."name is "..iname..", link is "..ilink..", roll opts "..lootTable[i].rollopts) --debug
		lootTable[i]["name"] = iname
		lootTable[i]["link"] = ilink
		subtype = subtype or class
		if equipSlot=="INVTYPE_CLOAK" then subtype="Cloak" end
		--self:Print(ilink..class.." "..subtype.." "..equipSlot)
		if string.find(lootTable[i].rollopts, "n") and FlamingLoot:DetermineAutopass(subtype, ilink) then
			lootTable[i].selec = "u"
		end  --TODO add func to come up with most relevant text
		lootTable[i]["subtype"] = subtype
		lootTable[i]["texture"] = texture
	end
	FlamingLoot:LootNextItem() 
end

function FlamingLoot:LootNextItem()
	FLLootRollFrameButton3:Hide()
	FLLootRollFrameButton4:Hide()
	if lootTable[currentitem].selec then
		FlamingLoot:RollonItem(lootTable[currentitem].selec)
	else
		local heightmult = 1
		--self:Print(lootTable[currentitem].name.." item num "..currentitem)
		FLLootRollFrameItem:SetScript("OnEnter", function(self, motion) FlamingLoot:ShowTooltip(FLLootRollFrameItem, lootTable[currentitem].link, "ANCHOR_RIGHT") end)
		FLLootRollFrameItem:SetScript("OnLeave", function(self, motion) GameTooltip:Hide() end)
		FLLootRollFrameItem:SetNormalTexture(lootTable[currentitem].texture)
		FLLootRollFrameItemLink:SetText(lootTable[currentitem].link)
		--FLLootRollFrameItemType:SetText(lootTable[currentitem].subtype)
		FLLootRollFrameButton1:SetText(optsdisplay[string.sub(lootTable[currentitem].rollopts, 1, 1)])
		FLLootRollFrameButton1:SetScript("OnClick", function() FlamingLoot:RollonItem(string.sub(lootTable[currentitem].rollopts, 1, 1)) end)
		FLLootRollFrameButton2:SetText(optsdisplay[string.sub(lootTable[currentitem].rollopts, 2, 2)])
		FLLootRollFrameButton2:SetScript("OnClick", function() FlamingLoot:RollonItem(string.sub(lootTable[currentitem].rollopts, 2, 2)) end)
		local opt = string.sub(lootTable[currentitem].rollopts, 3, 3)
		if  opt ~="" then
			heightmult = 2
			FLLootRollFrameButton3:SetText(optsdisplay[opt])
			FLLootRollFrameButton3:SetScript("OnClick", function() FlamingLoot:RollonItem(string.sub(lootTable[currentitem].rollopts, 3, 3)) end)
			FLLootRollFrameButton3:Show()
		end
		opt = string.sub(lootTable[currentitem].rollopts, 4, 4)
		if opt ~="" then
			FLLootRollFrameButton4:SetScript("OnClick", function() FlamingLoot:RollonItem(string.sub(lootTable[currentitem].rollopts, 4, 4)) end)
			FLLootRollFrameButton4:SetText(optsdisplay[opt])
			FLLootRollFrameButton4:Show()
		end
		FLLootRollFrame:SetHeight(70+30*heightmult)
		FLLootRollFrame:Show()
	end
end

function FlamingLoot:RollonItem(button)
	--"item name=rolltype;roll#"
	printstr=""
	lootTable[currentitem].msg = "roll:"..lootTable[currentitem].itemid.."="..button
	if button=="u" then printstr = "Auto-passed on "..lootTable[currentitem].link
	elseif button~="a" then printstr = "You rolled "..optsdisplay[button].." on "..lootTable[currentitem].link end 
	if button == "o" or button =="g" or button =="s" or button =="r" then
		local roll = tostring(math.random(100))
		printstr = printstr..", random roll: "..roll
		lootTable[currentitem].msg = lootTable[currentitem].msg..";"..roll
	end
	self:Print(printstr)
	SendAddonMessage("flamingloot", lootTable[currentitem].msg, "WHISPER", self.db.profile.masterlooter);
	FLLootRollFrame:Hide()
	wipe(lootTable[currentitem])
	if currentitem<lootcount then
		currentitem = currentitem+1
		if rollingopen==true then FlamingLoot:LootNextItem() end
	end
end

function FlamingLoot:EndLooting()
	FLLootRollFrame:Hide()
	rollingopen = false
	if currentitem<lootcount then 
		self:Print("**Loot rolling closed**")
		for i= currentitem, lootcount do --if looting was ended before we rolled on everything, clear the rest of the loottable
			FlamingLoot:RollonItem("a")
		end
	end
	lootcount = 0
	currentitem = 1
	wipe(lootTable)
end

--STANDINGS
function FlamingLoot:ShowStandings()
	if not RaiderStandingsText:GetText() then
		RaiderStandingsText:SetText(self.db.profile.standings)
		RaiderStandingsTime:SetText("updated "..date("%m/%d %I:%M%p", self.db.profile.standingstime))
	end
	RaiderStandings:Show()
end
     
function FlamingLoot:GetStandings()
	if time()-self.db.profile.standingstime>30 then
		SendAddonMessage("flamingsys", "standings:"..self.db.profile.standingstime, "WHISPER", self.db.profile.masterlooter)
	else
		self:Print("Please wait at least 30s between updates")
	end
end

function FlamingLoot:StandingsRecd(standings)
	RaiderStandingsText:SetText(standings)
	RaiderStandingsTime:SetText("updated "..date("%m/%d %I:%M%p", time()))
	self.db.profile.standingstime=time()
	self.db.profile.standings = standings
	--RaiderStandings:SetHeight(55+(12)*16)
end


--ADDON MESSAGE HANDLER
function FlamingLoot:CHAT_MSG_ADDON(event, prefix, message, channel, sender)
	if (prefix == "flamingloot") then
		local colon = strfind(message,":");
		local command = strsub(message,1, colon - 1)
		local param
		if #message>colon then 
			param = strsub(message, colon + 1)
		end
		if command == "loot" then
			FlamingLoot:StartLooting(sender,param)
		elseif command =="end" then
			FlamingLoot:EndLooting()
		elseif command=="standings" then
			FlamingLoot:StandingsRecd(param)
		elseif command=="uptodate" then
			FlamingLoot:Print("Your standings are up to date.")
			self.db.profile.standingstime=time()
		elseif command=="print" then
			FlamingLoot:Print(param)
		end
	end
end

--UTILITY FUNCS/LOOKUPS
function FlamingLoot:Print(text)
	if text then
		DEFAULT_CHAT_FRAME:AddMessage("|cff6600FFFL:|r "..text)
	end
end

local flower
function FlamingLoot:InconspicuousFunction()
	if not flower then
		flower = CreateFrame("PlayerModel", "Flower", UIParent)
		flower:SetPoint("CENTER")
		flower:SetCreature(52593)
		flower:SetRotation(math.pi/3)
		flower:EnableMouse(true)
		flower:SetScript("OnMouseDown", function(self, button)
			self:Hide()
		end)
	end
	flower:Show()
end

local disallowed = { --modified from LibGearPoints-1.2 (EPGP)
	Deathknight = {
		Leather = true,
		Mail = true,
		Cloth = true,
		Shields = true,
		Bows = true,
		Crossbows = true,
		Daggers = true,
		["Fist Weapons"] = true,
		Guns = true,
		Polearms = true,
		Staves = true,
		Idols = true,
		Librams = true,
		Thrown = true,
		Totems = true,
		Wands = true,
	},
	Druid = {
		Cloth = true,
		Mail = true,
		Plate = true,
		Plate = true,
		Shields = true,
		Bows = true,
		Crossbows = true,
		Guns = true,
		["One-Handed Axes"] = true,
		["One-Handed Swords"] = true,
		["Two-Handed Axes"] = true,
		["Two-Handed Swords"] = true,
		Librams = true,
		Sigils = true,
		Thrown = true,
		Totems = true,
		Wands = true,
	},
	Hunter= {
		Leather = true,
		Cloth = true,
		Plate = true,
		Shields = true,
		["One-Handed Maces"] = true,
		["Two-Handed Maces"] = true,
		Idols = true,
		Librams = true,
		Sigils = true,
		Totems = true,
		Wands = true,
	},
	Mage = {
		Leather = true,
		Mail = true,
		Plate = true,
		Shields = true,
		Bows = true,
		Crossbows = true,
		["Fist Weapons"] = true,
		Guns = true,
		["One-Handed Axes"] = true,
		["One-Handed Maces"] = true,
		Polearms = true,
		["Two-Handed Axes"] = true,
		["Two-Handed Maces"] = true,
		["Two-Handed Swords"] = true,
		Idols = true,
		Librams = true,
		Sigils = true,
		Thrown = true,
		Totems = true,
	},
	Paladin = {
		Leather = true,
		Mail = true,
		Cloth = true,
		Bows = true,
		Crossbows = true,
		["Fist Weapons"] = true,
		Guns = true,
		Staves = true,
		Idols = true,
		Sigils = true,
		Thrown = true,
		Totems = true,
		Wands = true,
	},
	Priest = {
		Leather = true,
		Mail = true,
		Plate = true,
		Shields = true,
		Bows = true,
		Crossbows = true,
		["Fist Weapons"] = true,
		Guns = true,
		["One-Handed Axes"] = true,
		["One-Handed Swords"] = true,
		Polearms = true,
		["Two-Handed Axes"] = true,
		["Two-Handed Maces"] = true,
		["Two-Handed Swords"] = true,
		Idols = true,
		Librams = true,
		Sigils = true,
		Thrown = true,
		Totems = true,
	},
	Rogue = {
		Cloth = true,
		Mail = true,
		Plate = true,
		Shields = true,
		Polearms = true,
		Staves = true,
		["Two-Handed Axes"] = true,
		["Two-Handed Maces"] = true,
		["Two-Handed Swords"] = true,
		Idols = true,
		Librams = true,
		Sigils = true,
		Totems = true,
		Wands = true,
		},
		Shaman = {
		Plate = true,
		Leather = true,
		Cloth = true,
		Bows = true,
		Crossbows = true,
		Guns = true,
		["One-Handed Swords"] = true,
		Polearms = true,
		["Two-Handed Swords"] = true,
		Idols = true,
		Librams = true,
		Sigils = true,
		Thrown = true,
		Wands = true,
	},
	Warlock = {
		Leather = true,
		Mail = true,
		Plate = true,
		Shields = true,
		Bows = true,
		Crossbows = true,
		["Fist Weapons"] = true,
		Guns = true,
		["One-Handed Axes"] = true,
		["One-Handed Maces"] = true,
		Polearms = true,
		["Two-Handed Axes"] = true,
		["Two-Handed Maces"] = true,
		["Two-Handed Swords"] = true,
		Idols = true,
		Librams = true,
		Sigils = true,
		Thrown = true,
		Totems = true,
	},
	Warrior = {
		Leather = true,
		Mail = true,
		Cloth = true,
		Idols = true,
		Librams = true,
		Sigils = true,
		Totems = true,
		Wands = true,
   },
 }

local PRIMARY_STAT = {
	Strength = {
		Deathknight = true,
		Paladin = true,
		Warrior = true,
	},
	Agility = {
		Druid = true,
		Hunter = true,
		Rogue = true,
		Shaman = true,
	},
	Intellect  = {
		Mage = true,
		Priest = true,
		Shaman = true,
		Warlock = true,
		Paladin = true,
		Druid = true,
	},
}
 
function FlamingLoot:IsMainStat(link) --adapted from LibItemUtils-1.0 (EPGP)
	FLScanningTooltip:SetOwner(UIParent, "ANCHOR_NONE")
	FLScanningTooltip:SetHyperlink(link)
	local section
	local stat
	for i=4, FLScanningTooltip:NumLines()-3 do
		section = getglobal("FLScanningTooltipTextLeft"..i)
		stat = string.match(section:GetText(), "%d+%s(%a+)")
		if stat and PRIMARY_STAT[stat] then
			return PRIMARY_STAT[stat][playerclass]
		end
	end
	return true
end

function FlamingLoot:DetermineAutopass(subtype, link) --returns true if autopass, false if not
	if subtype and disallowed[playerclass][subtype] then
		return true
	elseif link and (not FlamingLoot:IsMainStat(link)) then
		return true
	else
		return false
	end
end

function FlamingLoot:TextTooltip(owner, text)
	GameTooltip:SetOwner(owner, "ANCHOR_RIGHT")
	GameTooltip:SetText(text)
	GameTooltip:Show()
end

function FlamingLoot:ShowTooltip(owner, itemlink, position)
	GameTooltip:SetOwner(owner, position)
	GameTooltip:SetHyperlink(itemlink)
	GameTooltip:Show()
end